//------------------------------------------------------------------
#property copyright ""
#property link      ""
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 6
#property indicator_plots   2

#property indicator_label1  "HA smoothed wick high;HA smoothed wick low"
#property indicator_type1   DRAW_COLOR_HISTOGRAM2
#property indicator_color1  clrLimeGreen,clrPaleVioletRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
#property indicator_label2  "HA smoothed body high;HA smoothed body low"
#property indicator_type2   DRAW_COLOR_HISTOGRAM2
#property indicator_color2  clrLimeGreen,clrPaleVioletRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  3

//
//
//
//
//

enum enMaTypes
{
   mt_sma,   // Simple moving average
   mt_ema,   // Exponential moving average
   mt_smma,  // Smoothed MA
   mt_lwma   // Linear weighted MA
};

//
//
//
//
//

input int       HALength1       = 6;        // Price smoothing period
input enMaTypes HAType1         = mt_smma;  // Price smoothing method
input int       HALength2       = 2;        // Result smoothing period
input enMaTypes HAType2         = mt_lwma;  // Result smoothing method
input bool      AlertsOn        = false;    // Alerting 
input bool      AlertsOnCurrent = true;     // Alert on current bar
input bool      AlertsOnBody    = true;     // Alert on body color change
input bool      AlertsOnWick    = true;     // Alert on wick color change
input bool      AlertsMessage   = true;     // Show alert message
input bool      AlertsSound     = false;    // Play alert sound
input bool      AlertsEmail     = false;    // Send email message

//
//
//
//
//

double habh[];
double habl[];
double habc[];
double hawh[];
double hawl[];
double hawc[];

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,hawh,INDICATOR_DATA); 
   SetIndexBuffer(1,hawl,INDICATOR_DATA); 
   SetIndexBuffer(2,hawc,INDICATOR_COLOR_INDEX); 
   SetIndexBuffer(3,habh,INDICATOR_DATA); 
   SetIndexBuffer(4,habl,INDICATOR_DATA); 
   SetIndexBuffer(5,habc,INDICATOR_COLOR_INDEX); 
   return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workHa[][4];
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{
   if (ArrayRange(workHa,0)!= rates_total) ArrayResize(workHa,rates_total);

   //
   //
   //
   //
   //

   for (int i=(int)MathMax(prev_calculated-1,0); i<rates_total; i++)
   {
      double maOpen  = iMa(HAType1,open[i] ,HALength1,i,rates_total,0);
      double maClose = iMa(HAType1,close[i],HALength1,i,rates_total,1);
      double maHigh  = iMa(HAType1,high[i] ,HALength1,i,rates_total,2);
      double maLow   = iMa(HAType1,low[i]  ,HALength1,i,rates_total,3);

         //
         //
         //
         //
         //
         
         double haOpen;
         if (i>0)
                haOpen  = (workHa[i-1][2] + workHa[i-1][3])/2.0;
         else   haOpen  = maOpen+maClose;
         double haClose = (maOpen + maHigh + maLow + maClose) / 4.0;
         double haHigh  = MathMax(maHigh, MathMax(haOpen,haClose));
         double haLow   = MathMin(maLow , MathMin(haOpen,haClose));

         if(haOpen < haClose) { workHa[i][0] = haLow;  workHa[i][1] = haHigh; } 
         else                 { workHa[i][0] = haHigh; workHa[i][1] = haLow;  } 
                                workHa[i][2] = haOpen;
                                workHa[i][3] = haClose;
   
         //
         //
         //
         //
         //
         
         hawh[i] = iMa(HAType2,workHa[i][0],HALength2,i,rates_total,4);
         hawl[i] = iMa(HAType2,workHa[i][1],HALength2,i,rates_total,5);
         habh[i] = iMa(HAType2,workHa[i][2],HALength2,i,rates_total,6);
         habl[i] = iMa(HAType2,workHa[i][3],HALength2,i,rates_total,7);
         if (i>0)
         {
            hawc[i]=hawc[i-1];
            habc[i]=habc[i-1];
               if (hawh[i]>hawl[i]) hawc[i] = 1;
               if (hawh[i]<hawl[i]) hawc[i] = 0;
               if (habh[i]>habl[i]) habc[i] = 1;
               if (habh[i]<habl[i]) habc[i] = 0;
         }
   }
   manageAlerts(rates_total,time);
   return(rates_total);
}



//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts(int bars,const datetime& atime[])
{
   if (AlertsOn)
   {
      int      whichBar = 0; if (!AlertsOnCurrent) whichBar = 1; whichBar = bars-whichBar-1;
      datetime time     = atime[whichBar];
      
      //
      //
      //
      //
      //
      
      if (habc[whichBar] != habc[whichBar-1] && AlertsOnBody)
      {
         static datetime time1 = 0;
         static string   mess1 = "";
            if (habc[whichBar] == 0) doAlert(time1,mess1,time," HA body trend changed to up");
            if (habc[whichBar] == 1) doAlert(time1,mess1,time," HA body trend changed to down");
      }
      if (hawc[whichBar] != hawc[whichBar-1] && AlertsOnWick)
      {
         static datetime time2 = 0;
         static string   mess2 = "";
            if (hawc[whichBar] == 0) doAlert(time2,mess2,time," HA wick trend changed to up");
            if (hawc[whichBar] == 1) doAlert(time2,mess2,time," HA wick trend changed to down");
      }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert,datetime time, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != time) {
       previousAlert  = doWhat;
       previousTime   = time;

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToString(TimeLocal(),TIME_SECONDS)+doWhat;
          if (AlertsMessage) Alert(message);
          if (AlertsEmail)   SendMail(Symbol()+" Heiken ashi smoothed",message);
          if (AlertsSound)   PlaySound("alert2.wav");
   }
}


//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

#define _maWorkBufferx1  8
#define _maWorkBufferx2  16

double iMa(int mode, double price, double length, int r, int total, int instanceNo=0)
{
   switch (mode)
   {
      case mt_sma  : return(iSma(price,(int)length,r,total,instanceNo));
      case mt_ema  : return(iEma(price,length,r,total,instanceNo));
      case mt_smma : return(iSmma(price,length,r,total,instanceNo));
      case mt_lwma : return(iLwma(price,length,r,total,instanceNo));
      default : return(price);
   }
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workSma[][_maWorkBufferx2];
double iSma(double price, int period, int r,int bars, int instanceNo=0)
{
   if (ArrayRange(workSma,0)!= bars) ArrayResize(workSma,bars); instanceNo *= 2; int k;

   //
   //
   //
   //
   //
      
   workSma[r][instanceNo] = price;
   if (r>=period)
          workSma[r][instanceNo+1] = workSma[r-1][instanceNo+1]+(workSma[r][instanceNo]-workSma[r-period][instanceNo])/period;
   else { workSma[r][instanceNo+1] = 0; for(k=0; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo];  
          workSma[r][instanceNo+1] /= k; }
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][_maWorkBufferx1];
double iEma(double price, double period, int r,int bars, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= bars) ArrayResize(workEma,bars);

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

double workSmma[][_maWorkBufferx1];
double iSmma(double price, double period, int r, int bars, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= bars) ArrayResize(workSmma,bars);

   //
   //
   //
   //
   //

   if (r<period)
         workSmma[r][instanceNo] = price;
   else  workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workLwma[][_maWorkBufferx1];
double iLwma(double price, double period, int r, int bars, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= bars) ArrayResize(workLwma,bars);
   
   //
   //
   //
   //
   //
   
   workLwma[r][instanceNo] = price;
      double sumw = period;
      double sum  = period*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}